/**************************************************************************************************
* File: MP3_driver.c
* File Type: C - Source Code File
* (C) mikroElektronika, 2012
* Revision History:
*     - Initial version
* Description:
*     This project demonstrates communication with VS1053B mp3 codec.
*     Program reads one mp3 file from MMC and sends it to VS1053B for decoding
*     and playing.
*     MMC and MP3_SCI share Hardware SPI module.
* Test configuration:
*       MCU:             STM32F107VCT6
*                        http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
*       Dev.Board:       Custom
*       modules:         VS1053B MP3 Player
*                        http://www.mikroe.com/eng/products/view/783/mp3-click/
*       Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
*       SW:              mikroC PRO for ARM
*                        http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-arm/
* NOTES:
*     - Make sure that MMC card is properly formatted (to FAT32)
*       before testing it on this example.
*     - Make sure that MMC card contains appropriate mp3 file ( sound.mp3 ).
**************************************************************************************************/

#include "__Lib_FAT32.h"
#include "MP3_Driver.h"

// CODEC V1053E connections
sbit MP3_CS   at GPIOA_ODR.B4;
sbit MP3_RST  at GPIOE_ODR.B9;
sbit BSYNC    at GPIOE_ODR.B8;
sbit DREQ     at GPIOE_IDR.B7;

//LED GPIOE.5
sbit LED    at GPIOE_ODR.B5;

// MMC module connections
sbit Mmc_Chip_Select at GPIOB_ODR.B10;
sbit Mmc_Card_Detect at GPIOB_ODR.B11;    // for detecting microSD card presence

// variables
char Fat_Initialized_Flag;
const unsigned long mp3_note_bmp;
const unsigned long mp3_player_note_bmp;
char level;
signed short Ext_fhandle;
unsigned long Ext_StartSect;

const BYTES_2_WRITE = 32;

const unsigned long BUFFER_SIZE = 256;
const unsigned long BUFSIZE = 512;

char MP3_Example_State = 0;
unsigned long Buffer_count, Total_Buffers;
char MP3_song[13];
char current_fhandle;
//signed short current_file;
//short current_file;
char current_file;
int MP3_Example_cnt = 0;
//unsigned long file_size;
long file_size;
char mp3_buffer[BUFFER_SIZE]; //MP3 buffer as a char
//unsigned char *mp3_buffer[BUFFER_SIZE]; //MP3 buffer as a char

char volume_left, volume_right;

char Example_State;
char filename[] = "SO42.mp3";

__HANDLE fileHandle[4];

unsigned char HelloMP3[] = { // Note: "code" in AT89C51 SDCC means to put this array into ROM.
      0xFF,0xF2,0x40,0xC0,0x19,0xB7,0x00,0x14,0x02,0xE6,0x5C, /* ..@.......\ */
      0x01,0x92,0x68,0x01,0xF1,0x5E,0x03,0x08,0xF0,0x24,0x80, /* ..h..^...$. */
      0x05,0x9E,0x20,0xC6,0xFC,0x12,0x32,0x5C,0xBF,0xF9,0xB9, /* .. ...2\... */
      0x20,0x4A,0x7F,0x85,0xEC,0x4C,0xCD,0xC7,0x27,0xFE,0x5C, /*  J...L..'.\ */
      0x34,0x25,0xCB,0xE6,0xFF,0xFF,0x8E,0x42,0xE1,0xA0,0x5E, /* 4%.....B..^ */
      0xCA,0x6E,0x30,0x9F,0xFF,0xF8,0xC2,0x12,0x84,0xB9,0x7C, /* .n0.......| */
      0xDC,0x61,0x09,0x4A,0x7F,0xFF,0xFF,0xF9,0x7D,0x32,0x51, /* .a.J....}2Q */
      0x09,0x7C,0xE1,0xA5,0x6E,0xB4,0xFF,0xFF,0xFF,0xFF,0xD3, /* .|..n...... */
      0x34,0x41,0x91,0xF0,0x11,0x8F,0x00,0x0F,0x81,0x9C,0x10, /* 4A......... */
      0xEE,0x59,0xCE,0x56,0x67,0xFF,0xF2,0x42,0xC0,0xEC,0x53, /* .Y.Vg..B..S */
      0x09,0x15,0xF9,0xAA,0xA8,0x0D,0xD9,0x40,0x00,0xCA,0x34, /* .......@..4 */
      0x53,0xD9,0x18,0xAB,0x7D,0xF7,0x89,0x3F,0x11,0x38,0x94, /* S...}..?.8. */
      0x82,0x59,0x93,0x20,0x6A,0x0C,0xEE,0x8E,0x58,0xFA,0x38, /* .Y. j...X.8 */
      0x82,0xCA,0xF0,0x58,0xBB,0xDA,0x0C,0x50,0x56,0x1F,0xBB, /* ...X...PV.. */
      0x18,0x5D,0x8B,0x9F,0xDA,0x71,0x4F,0xFF,0xBD,0xFE,0xEF, /* .]...qO.... */
      0x69,0x36,0x86,0x3C,0x50,0xBB,0x0A,0x07,0x89,0x54,0xF0, /* i6.<P....T. */
      0x88,0x9F,0x90,0x95,0x30,0x94,0x2E,0x7E,0xF0,0x64,0x96, /* ....0..~.d. */
      0x79,0x08,0x3E,0x20,0x97,0x28,0x34,0x9C,0x09,0x7F,0xD2, /* y.> .(4.... */
      0xC0,0x01,0x75,0xF8,0x05,0x6B,0x5F,0x41,0x17,0x0B,0xE7, /* ..u..k_A... */
      0xFF,0xF2,0x40,0xC0,0x61,0xE5,0x0B,0x16,0x09,0xC6,0xC5, /* ..@.a...... */
      0x74,0x7B,0xCC,0x94,0x7A,0xF7,0x80,0x76,0xB2,0xD2,0xF8, /* t{..z..v... */
      0x39,0x06,0x38,0xFD,0x71,0xC5,0xDE,0x3A,0x38,0xBF,0xD5, /* 9.8.q..:8.. */
      0xF7,0x12,0x37,0xCB,0xF5,0x63,0x0C,0x9B,0xCE,0x77,0x25, /* ..7..c...w% */
      0xED,0xFB,0x3D,0x6B,0x35,0xF9,0x6D,0xD7,0xF9,0x2C,0xD1, /* ..=k5.m..,. */
      0x97,0x15,0x87,0x93,0xA4,0x49,0x4A,0x18,0x16,0x07,0xA1, /* .....IJ.... */
      0x60,0xF7,0x52,0x94,0xDB,0x02,0x16,0x70,0xB2,0xD8,0x80, /* `.R....p... */
      0x30,0xC2,0x94,0x40,0x81,0x74,0x5A,0x19,0x7A,0x80,0x60, /* 0..@.tZ.z.` */
      0x41,0x21,0x46,0x95,0xD5,0xC4,0x40,0xD2,0x01,0xC0,0x01, /* A!F...@.... */
      0xDA,0xD9,0xA0,0xB1,0x01,0xFF,0xF2,0x42,0xC0,0x82,0x10, /* .......B... */
      0x0B,0x12,0xF9,0x9E,0xC9,0x7E,0x7A,0xC6,0x95,0x55,0x09, /* .....~z..U. */
      0x8B,0x19,0x5E,0x8B,0x26,0xCA,0xEB,0x68,0x8A,0x05,0x8F, /* ..^.&..h... */
      0x36,0xA5,0xA5,0x03,0xB8,0x9C,0xED,0x24,0x51,0x59,0x90, /* 6......$QY. */
      0xF6,0xC5,0x7D,0xB5,0xAD,0xAF,0xF6,0x3B,0x18,0xEF,0x3F, /* ..}....;..? */
      0xFF,0xFF,0x4E,0xDE,0x16,0x66,0x0B,0xAA,0x33,0x23,0xDD, /* ..N..f..3#. */
      0x9C,0x4E,0x6E,0x55,0x22,0x9D,0xA2,0x40,0xA6,0x36,0x31, /* .NnU"..@.61 */
      0x69,0xA5,0xE1,0xD9,0x7F,0xF7,0xC6,0xCC,0x48,0x00,0x0E, /* i.......H.. */
      0x90,0x16,0x00,0x0F,0xDE,0x6E,0x80,0x11,0x0C,0x9A,0x4F, /* .....n....O */
      0x56,0xDB,0x88,0xD3,0xB2,0x1C,0x00,0xE0,0x2E,0x3E,0xAC, /* V........>. */
      0xFF,0xF2,0x40,0xC0,0x1C,0xE5,0x19,0x13,0x31,0x4E,0xCD, /* ..@.....1N. */
      0x9E,0xC3,0x06,0x71,0x03,0x85,0xE5,0xB5,0x6D,0x88,0x50, /* ...q....m.P */
      0x8E,0x0E,0x17,0x3B,0x19,0xFB,0x4E,0x3B,0x99,0xEF,0x4C, /* ...;..N;..L */
      0x9E,0xF7,0x7B,0x31,0x7C,0x3C,0x5F,0xFF,0xF4,0xF8,0xE3, /* ..{1|<_.... */
      0x92,0x42,0x07,0x8E,0x83,0x8E,0x0F,0x05,0x08,0x91,0xA3, /* .B......... */
      0x16,0xE2,0xDF,0xB7,0x62,0x60,0x48,0x31,0x3C,0xFF,0xD4, /* ....b`H1<.. */
      0x9E,0x0C,0x68,0x00,0x77,0x54,0xE3,0x1E,0x05,0xC5,0xF8, /* ..h.wT..... */
      0xEA,0x8D,0x82,0x9D,0x08,0xA9,0x06,0x8D,0x1E,0x5D,0x7C, /* .........]| */
      0x7F,0x08,0xC0,0x50,0x45,0x42,0xD0,0x36,0xF8,0xB2,0x4D, /* ...PEB.6..M */
      0x53,0x0C,0x80,0x3B,0x4D,0xFF,0xF2,0x42,0xC0,0x2F,0x3C, /* S..;M..B./< */
      0x25,0x19,0x29,0xFE,0xBC,0x2E,0xC4,0xD0,0x99,0x4C,0x48, /* %.)......LH */
      0xB0,0x9C,0x49,0xD2,0x1A,0x2D,0x02,0xC2,0x79,0x69,0x16, /* ..I..-..yi. */
      0x92,0xA8,0xC5,0xAB,0x45,0x5A,0x68,0xE8,0x75,0x57,0xCD, /* ....EZh.uW. */
      0xF1,0xB9,0xAA,0x13,0x88,0xE4,0x87,0x42,0x15,0xB3,0x58, /* .......B..X */
      0xF5,0xA3,0x46,0xB1,0xCF,0xD3,0x59,0x7E,0xBA,0xB5,0xA7, /* ..F...Y~... */
      0x6B,0x0B,0x17,0x57,0x6B,0x5C,0x4A,0xCD,0x53,0x76,0x2A, /* k..Wk\J.Sv* */
      0x1D,0x28,0xC5,0x1C,0x76,0x5C,0xDD,0x0A,0x00,0x4B,0xC0, /* .(..v\...K. */
      0x1B,0xCA,0xA8,0xE9,0x81,0x5B,0xA6,0xDC,0xA4,0x59,0x13, /* .....[...Y. */
      0xFC,0xBA,0x8F,0x98,0x79,0x44,0x25,0xC9,0x35,0x38,0xCA, /* ....yD%.58. */
      0xFF,0xF2,0x40,0xC0,0xB9,0x7D,0x1A,0x13,0x79,0x6A,0xC8, /* ..@..}..yj. */
      0x3E,0xC4,0x46,0x94,0x8D,0x3C,0x67,0x85,0xB1,0xA8,0x89, /* >.F..<g.... */
      0xC0,0xF2,0xE6,0x2F,0x9D,0x7C,0xC9,0xB4,0xBE,0xCF,0xE1, /* .../.|..... */
      0x7D,0xFE,0x1F,0x03,0x00,0x12,0x84,0x72,0x8C,0xE7,0xD8, /* }......r... */
      0x5E,0xC9,0xA9,0x01,0xBA,0x9B,0xC4,0x10,0x5C,0x70,0x2E, /* ^.......\p. */
      0x6C,0x48,0xE7,0x8C,0x15,0x0B,0x06,0x01,0xE5,0xFF,0xFF, /* lH......... */
      0xD4,0x0D,0x00,0x0F,0xCE,0x58,0x95,0x61,0xA8,0x9E,0x7B, /* .....X.a..{ */
      0x19,0x98,0xB0,0xF0,0xC6,0x72,0x82,0xD5,0x27,0x06,0x47, /* .....r..'.G */
      0x41,0x22,0x0F,0x65,0x93,0xC9,0x8A,0x09,0x19,0x48,0x1B, /* A".e.....H. */
      0xBD,0xD6,0x64,0x1A,0xAC,0xFF,0xF2,0x42,0xC0,0xF1,0x11, /* ..d....B... */
      0x25,0x14,0x22,0x06,0xBC,0x0E,0xD4,0x4E,0x99,0x90,0xA8, /* %."....N... */
      0xD8,0xB7,0xAD,0x5D,0x3E,0xAF,0x6E,0xBE,0x66,0x83,0xA4, /* ...]>.n.f.. */
      0xE3,0xC2,0xE0,0x29,0x43,0x87,0x5F,0x4F,0x27,0x9C,0x2C, /* ...)C._O'., */
      0xD0,0x91,0xF3,0x87,0x9B,0x54,0xED,0xD1,0xB4,0xF3,0x39, /* .....T....9 */
      0x87,0x22,0x06,0x86,0x0D,0x71,0xE4,0x6F,0x2A,0x08,0x04, /* ."...q.o*.. */
      0xC0,0x03,0x2A,0xB1,0xE2,0x05,0x4D,0x64,0xA1,0x9C,0xA6, /* ..*...Md... */
      0x0D,0x41,0xA6,0xF2,0x7A,0xC1,0x30,0xC3,0x38,0x26,0x09, /* .A..z.0.8&. */
      0x50,0x08,0xC4,0xF6,0x30,0x0C,0xA6,0xA9,0x17,0x00,0x13, /* P...0...... */
      0x0C,0xDC,0xC4,0x2F,0x28,0xEB,0x3F,0xCD,0x7A,0x3D,0x2F, /* .../(.?.z=/ */
      0xFF,0xF2,0x40,0xC0,0x18,0x6F,0x2E,0x13,0xA1,0xF2,0xBC, /* ..@..o..... */
      0x36,0xCB,0x4E,0x99,0x6E,0xFC,0xEE,0xC5,0xF0,0xA0,0xB7, /* 6.N.n...... */
      0x92,0xD4,0xEE,0x79,0x7C,0x50,0x5D,0xE5,0x04,0x94,0xA9, /* ...y|P].... */
      0x76,0xCF,0x6C,0x70,0xDD,0x0D,0xD4,0xEE,0xED,0x98,0xE8, /* v.lp....... */
      0xC8,0x35,0x36,0x7A,0x0C,0x05,0x80,0x03,0xBC,0xBE,0x91, /* .56z....... */
      0x00,0x7C,0xAE,0x65,0xB8,0x91,0xA3,0x33,0xBA,0x68,0x60, /* .|.e...3.h` */
      0xD4,0x1A,0x66,0xF8,0x43,0xA0,0x20,0x89,0xE7,0x80,0xD8, /* ..f.C. .... */
      0x1E,0x4F,0xA0,0x04,0x60,0x06,0x0A,0xA4,0x91,0x24,0xFA, /* .O..`....$. */
      0x9F,0x57,0x53,0xF4,0x7A,0xDB,0x5F,0x56,0xE3,0x6E,0x0B, /* .WS.z._V.n. */
      0x8B,0x3A,0x1C,0xF9,0x5E,0xFF,0xF2,0x42,0xC0,0xB1,0x00, /* .:..^..B... */
      0x38,0x14,0x09,0xEE,0xB4,0x36,0xD3,0x4E,0x99,0xA4,0x78, /* 8....6.N..x */
      0x94,0x73,0xC4,0x66,0x30,0xF5,0xEA,0xDB,0xBA,0x67,0x67, /* .s.f0....gg */
      0x95,0x6B,0xAB,0x68,0x5D,0x08,0xA1,0x39,0x56,0xAB,0x1E, /* .k.h]..9V.. */
      0xD5,0x03,0xE8,0x01,0x70,0x00,0xB3,0x93,0x33,0x19,0x8C, /* ....p...3.. */
      0x61,0x8F,0xBB,0x5D,0x24,0x12,0x63,0xD3,0x4B,0x5D,0x91, /* a..]$.c.K]. */
      0x08,0x43,0x22,0x56,0x1A,0xC5,0x10,0x21,0x84,0xA8,0xEA, /* .C"V...!... */
      0x80,0xBF,0x16,0x8E,0x3D,0x46,0x18,0x9C,0x6E,0x9A,0x91, /* ....=F..n.. */
      0xE6,0xC9,0x6F,0xD2,0x7D,0x27,0xD7,0xE9,0x6B,0xFF,0x0A, /* ..o.}'..k.. */
      0x03,0x43,0x89,0xD5,0xBF,0x52,0x97,0x0A,0x25,0x95,0x0D, /* .C...R..%.. */
      0xFF,0xF2,0x40,0xC0,0xF5,0xC3,0x41,0x13,0x81,0xEE,0xA8, /* ..@...A.... */
      0x5E,0xD3,0x44,0x98,0xFC,0xCF,0x97,0xF9,0x58,0xB5,0x33, /* ^.D.....X.3 */
      0xB1,0x85,0x47,0x86,0xD7,0x98,0x01,0x3B,0xA3,0x4F,0x7E, /* ..G....;.O~ */
      0x04,0xA6,0xC3,0x39,0x21,0x70,0x27,0x62,0xB5,0x18,0x10, /* ...9!p'b... */
      0x09,0x99,0x00,0x8B,0x7E,0xF2,0xBF,0x52,0x18,0x26,0x30, /* ....~..R.&0 */
      0x1C,0xB0,0x01,0x49,0x30,0xE0,0xC3,0x11,0x46,0x05,0xCC, /* ...I0...F.. */
      0x49,0x14,0x28,0xB2,0xED,0x4B,0x57,0x5A,0x2F,0xB7,0x46, /* I.(..KWZ/.F */
      0x63,0x34,0xD2,0xDA,0x9F,0x56,0x32,0xB7,0xA2,0x25,0xFF, /* c4...V2..%. */
      0x94,0x28,0x33,0x7F,0x3B,0xC4,0x50,0xEC,0xB1,0xE2,0x26, /* .(3.;.P...& */
      0xA1,0xB7,0x07,0x7F,0xFB,0xFF,0xF2,0x42,0xC0,0x67,0x6A, /* .......B.gj */
      0x4C,0x13,0xF9,0x6A,0x90,0x7E,0xDB,0x44,0x94,0x3F,0xFF, /* L..j.~.D.?. */
      0x14,0xD6,0x2A,0xFF,0xFF,0xC1,0x34,0x8C,0x48,0x22,0x00, /* ..*...4.H". */
      0x06,0x8F,0x21,0xFD,0x64,0x60,0x04,0x92,0x42,0xEA,0x74, /* ..!.d`..B.t */
      0x32,0x37,0xAA,0x5A,0x9F,0x67,0x01,0x8B,0x3F,0x37,0x31, /* 27.Z.g..?71 */
      0xDD,0x06,0x3C,0x01,0x34,0x30,0xE0,0x5C,0x78,0x78,0xCB, /* ..<.40.\xx. */
      0xD6,0xF1,0x31,0x8A,0x69,0x61,0x93,0x92,0x42,0xCE,0x4B, /* ..1.ia..B.K */
      0xC5,0x02,0x4E,0x73,0xC6,0x24,0x30,0xCD,0x08,0x66,0xC6, /* ..Ns.$0..f. */
      0x35,0xAB,0xA2,0x3D,0x2F,0xB3,0xBD,0x34,0x87,0x13,0xEE, /* 5..=/..4... */
      0x71,0x45,0x68,0xFA,0xEA,0x05,0x84,0x41,0x36,0x4C,0x9A, /* qEh....A6L. */
      0xFF,0xF2,0x40,0xC0,0xC9,0x92,0x56,0x13,0xD0,0x6E,0x70, /* ..@...V..np */
      0x54,0xD3,0xCC,0x28,0x06,0xD7,0x0E,0xA4,0x1D,0x9C,0x9D, /* T..(....... */
      0xD9,0xA9,0x88,0x7B,0xB5,0xA3,0x56,0xB7,0x4B,0x4B,0x5A, /* ...{..V.KKZ */
      0x9B,0x2C,0xA9,0xAD,0x6F,0x99,0x6C,0xC0,0x4C,0x14,0x14, /* .,..o.l.L.. */
      0xEF,0xB4,0x20,0x91,0x5F,0xBC,0x81,0x41,0x41,0x5D,0xD4, /* .. ._..AA]. */
      0x20,0xBD,0x05,0x1A,0x6F,0xE2,0x68,0x56,0x41,0x41,0x57, /*  ...o.hVAAW */
      0xF9,0xBF,0x89,0x82,0x8E,0xC7,0x8F,0x0A,0x0A,0x09,0x37, /* ..........7 */
      0xF1,0x05,0x0A,0x0A,0x0A,0x0A,0x09,0x05,0x37,0xFF,0x10, /* ........7.. */
      0x50,0x50,0x53,0x65,0xFF,0xFF,0xFD,0x75,0xDF,0xFF,0xFF, /* PPSe...u... */
      0x68,0x4F,0xFF,0x84,0x70,0xFF,0xF2,0x42,0xC0,0x27,0x50, /* hO..p..B.'P */
      0x5F,0x17,0xE8,0x82,0x3C,0x11,0x58,0x18,0x01,0x55,0x48, /* _...<.X..UH */
      0xBC,0x52,0xFC,0x4A,0x4C,0x3C,0xD5,0xF6,0x11,0x2D,0xBF, /* .R.JL<...-. */
      0xEA,0x03,0x5C,0x57,0x29,0xBF,0xC3,0x75,0x1C,0xE6,0xDD, /* ..\W)..u... */
      0xBF,0xED,0xEF,0xD0,0x98,0x77,0x71,0x95,0x73,0xFF,0xED, /* .....wq.s.. */
      0x54,0xBE,0xD5,0xEE,0xAE,0xC2,0xD5,0x0B,0xFF,0xF1,0x97, /* T.......... */
      0x8A,0xE4,0x42,0x09,0x99,0xB1,0xEA,0x94,0xDC,0x78,0xB5, /* ..B......x. */
      0x34,0x0F,0xF1,0x8F,0xFC,0x15,0xF6,0xFA,0xB1,0x47,0xA9, /* 4........G. */
      0x6C,0x67,0x43,0x8B,0xF2,0x76,0x22,0xED,0xDA,0x85,0xBA, /* lgC..v".... */
      0x2F,0xC7,0xF9,0xCF,0xFC,0xDB,0x46,0x2E,0x50,0x0A,0x84, /* /.....F.P.. */
      0xFF,0xF2,0x40,0xC0,0xC6,0x4A,0x59,0x28,0x2B,0x19,0xE0, /* ..@..JY(+.. */
      0x01,0x89,0x78,0x00,0x52,0x85,0x3C,0x8E,0x54,0x9A,0x48, /* ..x.R.<.T.H */
      0x5A,0x72,0x32,0x94,0xBF,0x43,0x4F,0x24,0x53,0x4B,0xEC, /* Zr2..CO$SK. */
      0x4B,0x99,0x0E,0x66,0x1F,0xFF,0xCE,0x7F,0xFF,0x3F,0x10, /* K..f.....?. */
      0xAE,0x82,0x62,0x71,0x34,0x18,0x59,0x9B,0x51,0xC7,0x59, /* ..bq4.Y.Q.Y */
      0xCE,0xEE,0xA5,0xFE,0x02,0xBB,0x30,0x91,0x49,0xD5,0x4B, /* ......0.I.K */
      0xF3,0xDC,0x9A,0xA9,0x57,0x8E,0x72,0x10,0xC0,0x5D,0x60, /* ....W.r..]` */
      0x67,0xFC,0x7D,0xD6,0xBA,0xDD,0xB3,0x8B,0x5A,0x0A,0x4C, /* g.}.....Z.L */
      0x41,0x4D,0x45,0x33,0x2E,0x39,0x33,0xAA,0xAA,0xAA,0xAA, /* AME3.93.... */
      0xAA,0xAA,0xAA,0xAA,0xAA,0x54,0x41,0x47,0x48,0x65,0x6C, /* .....TAGHel */
      0x6C,0x6F,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20, /* lo          */
      0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20, /*             */
      0x20,0x20,0x20,0x20,0x20,0x50,0x61,0x6E,0x75,0x2D,0x4B, /*      Panu-K */
      0x72,0x69,0x73,0x74,0x69,0x61,0x6E,0x20,0x50,0x6F,0x69, /* ristian Poi */
      0x6B,0x73,0x61,0x6C,0x6F,0x20,0x20,0x20,0x20,0x20,0x20, /* ksalo       */
      0x20,0x20,0x56,0x53,0x44,0x53,0x50,0x20,0x54,0x65,0x73, /*   VSDSP Tes */
      0x74,0x69,0x6E,0x67,0x20,0x20,0x20,0x20,0x20,0x20,0x20, /* ting        */
      0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20, /*             */
      0x20,0x20,0x20,0x4D,0x50,0x33,0x20,0x48,0x65,0x6C,0x6C, /*    MP3 Hell */
      0x6F,0x2C,0x20,0x57,0x6F,0x72,0x6C,0x64,0x21,0x20,0x20, /* o, World!   */
      0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20, /*             */
      0x00, /* . */
    };

void DrawFrame();
char Init_FAT();
void MP3_Init();
void MP3_Start();
void MP3_play(char *Song_Name);
void StopMP3_File();
void doMP3(char *filename);

/*******************************************************************************
* Function Init_FAT()
* ------------------------------------------------------------------------------
* Overview: Function Initialize FAT32 system
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Init_FAT() {
    char FAT_cnt = 0;

    if (Fat_Initialized_Flag == 0) {
         SPI1_Init_Advanced(_SPI_FPCLK_DIV128, _SPI_MASTER  | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI1_PA567);
         SPI_Set_Active(SPI1_Read, SPI1_Write);

        while ((Fat32_Init() != 0) && (FAT_cnt < 5))
            Fat_cnt ++;
        
        if (FAT_cnt < 5){
            Fat_Initialized_Flag = 1;

             //SPI1_CR1 = 0;
              SPI1_Init_Advanced(_SPI_FPCLK_DIV64, _SPI_MASTER  | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI1_PA567);
             SPI_Set_Active(SPI1_Read, SPI1_Write);
             UART1_Write_Text("1. FAT32 Initialized\n");
            //TFT_Write_Text("1. FAT32 Initialized", 90, 55);
        }
    }

}

/*******************************************************************************
* Function StopMP3_File()
* ------------------------------------------------------------------------------
* Overview: Function stops playing and closes mp3 file
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void StopMP3_File() {
    FAT32_Close(current_fhandle);
    //MP3_Example_State = 3;
}
/*******************************************************************************
* Function Hello()
* ------------------------------------------------------------------------------
* Overview: Function Hello
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void hello(void)
{
   unsigned char *p;
    int i;
   
   p = HelloMP3;
   while (p <= &HelloMP3[sizeof(HelloMP3)-1]) 
   {
      while (DREQ == 0);
      BSYNC = 1;
      BSYNC = 0;
      MP3_SDI_Write_32(p);
      p += 32;
   }
   // End of file - send 2048 zeros before next file
    BSYNC = 0;

    for (i=0; i<2048; i++) 
    {
       while (DREQ == 0); // wait here until DREQ is high again
        MP3_SDI_Write(0x00);
      }
        Delay_ms(10);
                //SPIWait(); // Wait until SPI transfer is completed
    BSYNC = 1;
    UART_Write_Text("Hello Function Finished!\n");
}

/*******************************************************************************
* Function Read_Play_MP3()
* ------------------------------------------------------------------------------
* Overview: Function reading file
* Input: Nothing
* Output: Nothing
*******************************************************************************/
 void Read_Play_MP3()
{

    unsigned long i, fsize;
    unsigned int j, cnt,count;
    short fhandle;
    //char fhandle;
    char readBuffer[BUFSIZE];
    char str[55];

             LED = 0;

    fhandle = FAT32_Open("hello.mp3", FILE_READ);
    if (fhandle >= 0)
    {
          
          UART1_Write_Text("File test1.mp3 opened sucessfully!\n");
          FAT32_Size("hello.mp3", &fsize);

          LongToStr(fsize, str);
          UART_Write_Text("hello.mp3 size: ");
          UART_Write_Text(str);
          UART_Write(CR);
          
          while (FAT32_Eof(fhandle) != 1)
          //while (fsize >= BUFSIZE)
          {
            //Mmc_Chip_Select = 0;
            FAT32_Read(fhandle, readBuffer, BUFSIZE);
                     count =0;
                       //while(count<fsize)
                        while(count<512)
                        {
                           for(j=0;j<32;j++)
                            {


                                 if (DREQ == 1)
                                 {
                                   //Delay_ms(50);
                                   MP3_SDI_Write(readBuffer[count]);
                                   //UART_Write(readBuffer[count]);
                                   count++;
                                   //fsize -= BUFSIZE;
                                  }
                            }
                            //Delay_ms(10);
                        }
                 LED = 1;
                 Delay_ms(100);
                 LED = 0;
           }
                 UART1_Write_Text("\nDone  !\n DONE! \n");
                 LED = 1;
                 Delay_ms(1000);
                 LED = 0;
                 FAT32_Close(fhandle);
         
     }
     else
     {
            UART1_Write_Text("Opening file hello.mp3 failure!\n");
     }


 }
/*******************************************************************************
* Function Test_Read()
* ------------------------------------------------------------------------------
* Overview: Function reading file
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Test_Read()
{
    unsigned long i, fsize;
    unsigned int j, cnt;
    short fhandle;
    //char *ReadBuffer[100];
     char ReadBuffer[32];
    char str[55];


    
    fhandle = FAT32_Open("readme.txt", FILE_READ);
    FAT32_Size("readme.txt", &fsize);
    

    LongToStr(fsize, str);
    UART_Write_Text("readme.txt size:\n");
    UART_Write_Text(str);
    UART_Write(CR);
    
    for (i = 0; i < fsize; /**/) 
      {  
        FAT32_Read(fhandle, ReadBuffer, 32);
        for (j = 0; j < 32; i++, j++)
          {
             //if (i < fsize)
              // UART_Write_Text("File content:/n");
               UART_Write(ReadBuffer[j]);
               // MP3_SDI_Write_32(ReadBuffer[j]);
           }

      }
    UART_Write(CR);
    FAT32_Close(fhandle);



}

/*******************************************************************************
* Function PlayMP3()
* ------------------------------------------------------------------------------
* Overview: Function runs mp3 example. Should be called frequently
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void PlayMP3()
{

    unsigned long i;
    char str_file_size[22],str[22];
    char buff_size[22];
    
    Mmc_Chip_Select = 0;
    MP3_CS = 1;
    BSYNC = 1;
    
    current_file = FAT32_Open("test1.mp3", FILE_READ);
  if (current_file >= 0 )      // does the file open properly ?
  {

    UART_Write_Text("Play_MP3 function test1.mp3 opened..\n");
    FAT32_Size("test1.mp3",&file_size);
                     while (file_size > BUFFER_SIZE)
                     {
                        for (i=0; i<BUFFER_SIZE; i++)
                         {
                           Fat32_Read(current_file, mp3_buffer + i, BUFFER_SIZE);
                         }
                         


                         for (i = 0; i < BUFFER_SIZE / BYTES_2_WRITE; i++)
                         {
                           MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
                         }

                         file_size -= BUFFER_SIZE;

                         


                        LED = 1;
                     }

                     while (file_size < BUFFER_SIZE)
                     {


                         Fat32_Read(current_file, mp3_buffer, file_size);

                         for (i = 0; i < file_size; i++){
                           MP3_SDI_Write(mp3_buffer[i]);


                         }
                         LED =0;

                     }

                       UART_Write_Text("PlayMP3 Function Finished!\n");
                    FAT32_Close(current_file);
   }
   else
   {
      UART_Write_Text("Failed opening file!\n");
   }
}



/*******************************************************************************
* Function Mp3_Init()
* ------------------------------------------------------------------------------
* Overview: Function Starts MP3 example (draws appropriate Screen and sets state)
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MP3_Init() {
    Init_FAT();
    MP3_Start();
}

/*******************************************************************************
* Function MP3_Start()
* ------------------------------------------------------------------------------
* Overview: Function Initialize SPI and MP3 codec chip for operation
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void MP3_Start() {

    // MP3_CS, MP3_RST and BSYNC are outputs
    MP3_CS            = 1;               // Deselect MP3_CS
    MP3_RST           = 1;               // Set MP3_RST pin
    BSYNC             = 0;               // Clear BSYNC
    BSYNC             = 1;               // Clear BSYNC

    // Initialize SPI1 module

    SPI1_Init_Advanced(_SPI_FPCLK_DIV64, _SPI_MASTER  | _SPI_8_BIT |
                     _SPI_CLK_IDLE_LOW | _SPI_FIRST_CLK_EDGE_TRANSITION |
                     _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE | _SPI_SSI_1,
                     &_GPIO_MODULE_SPI1_PA567);
      SPI_Set_Active(SPI1_Read, SPI1_Write);

    MP3_Set_Default_Mode();

    Delay_ms(200);
    level = 0;
    MP3_Set_Volume(level, level);
    MP3_Reset_Time();
    UART1_Write_Text("2. MP3 Initialized\n");

}



void main() {

    // MP3 config
    GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);     // RST
    GPIO_Digital_Input (&GPIOE_BASE, _GPIO_PINMASK_7);     // DREQ
    GPIO_Digital_Output(&GPIOA_BASE, _GPIO_PINMASK_4);    // CS
    GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_8);    // DCS
    
    //LED
    GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_5);    // LED

    // MMC module connections
    GPIO_Digital_Output(&GPIOB_BASE, _GPIO_PINMASK_10);    // Mmc_Chip_Select
    //GPIO_Digital_Input(&GPIOB_BASE, _GPIO_PINMASK_11);    // Mmc_Card_Detect
  

     UART1_Init(115200);
    // Set default values for flags
    Fat_Initialized_Flag = 0;
    Ext_fhandle = 0xFF;
    Example_State = 0;
    UART1_Write_Text("MP3 CHIP AND SDCARD TEST STARTS\n");
    UART1_Write_Text("DIAGNOSTIC MESSAGE : \n");
     MP3_Init();

    while(1) {

      // Delay_ms(1000);
      //VsSineTest();
      // Delay_ms(1000);
      //Hello();
      //Delay_ms(1000);
      //Test_Read();
      //PlayMP3();
      Delay_ms(1000);

       Read_Play_MP3();

    }
}