/**************************************************************************************************
* File: MP3_driver.c
* File Type: C - Source Code File
* (C) mikroElektronika, 2012
* Revision History:
*     - Initial version
* Description:
*     This project demonstrates communication with VS1053B mp3 codec.
*     Program reads one mp3 file from MMC and sends it to VS1053B for decoding
*     and playing.
*     MMC and MP3_SCI share Hardware SPI module.
* Test configuration:
*       MCU:             STM32F107VCT6
*                        http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
*       Dev.Board:       Custom
*       modules:         VS1053B MP3 Player
*                        http://www.mikroe.com/eng/products/view/783/mp3-click/
*       Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
*       SW:              mikroC PRO for ARM
*                        http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-arm/
* NOTES:
*     - Make sure that MMC card is properly formatted (to FAT32)
*       before testing it on this example.
*     - Make sure that MMC card contains appropriate mp3 file ( sound.mp3 ).
**************************************************************************************************/

/**************************************************************************************************
* File: MP3_driver.c
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2011-2011
* Description:
*     This module contains a set of functions that are used for communication with
*     VS1053E mp3 codec.
* NOTES:
*     Mp3 codec use SPI bus to communicate with MCU.
**************************************************************************************************/
#include <built_in.h>
#include "MP3_Driver.h"

// VS1053E constants
const char WRITE_CODE           = 0x02;
const char READ_CODE            = 0x03;

const char SCI_BASE_ADDR        = 0x00;
const char SCI_MODE_ADDR        = 0x00;
const char SCI_STATUS_ADDR      = 0x01;
const char SCI_BASS_ADDR        = 0x02;
const char SCI_CLOCKF_ADDR      = 0x03;
const char SCI_DECODE_TIME_ADDR = 0x04;
const char SCI_AUDATA_ADDR      = 0x05;
const char SCI_WRAM_ADDR        = 0x06;
const char SCI_WRAMADDR_ADDR    = 0x07;
const char SCI_HDAT0_ADDR       = 0x08;
const char SCI_HDAT1_ADDR       = 0x09;
const char SCI_AIADDR_ADDR      = 0x0A;
const char SCI_VOL_ADDR         = 0x0B;
const char SCI_AICTRL0_ADDR     = 0x0C;
const char SCI_AICTRL1_ADDR     = 0x0D;
const char SCI_AICTRL2_ADDR     = 0x0E;
const char SCI_AICTRL3_ADDR     = 0x0F;

/**************************************************************************************************
* Function MP3_SCI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function writes one byte to MP3 SCI
* Input: register address in codec, data
* Output: Nothing
**************************************************************************************************/
void MP3_SCI_Write(char address, unsigned int data_in) {
    BSYNC = 1;

    MP3_CS = 0;                    // select MP3 SCI
    SPI1_Write(WRITE_CODE);
    SPI1_Write(address);
    SPI1_Write(Hi(data_in));        // high byte
    SPI1_Write(Lo(data_in));        // low byte
    MP3_CS = 1;                    // deselect MP3 SCI
    
    while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SCI_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function reads words_count words from MP3 SCI
* Input: start address, word count to be read
* Output: words are stored to data_buffer
**************************************************************************************************/
void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
    unsigned int temp;

    MP3_CS = 0;                    // select MP3 SCI
    SPI1_Write(READ_CODE);
    SPI1_Write(start_address);

    while (words_count--) {        // read words_count words byte per byte
        temp = SPI_Read(0);
        temp <<= 8;
        temp += SPI_Read(0);
        *(data_buffer++) = temp;
    }
    
    MP3_CS = 1;                    // deselect MP3 SCI
    
    while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

/**************************************************************************************************
* Function MP3_SDI_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write one byte to MP3 SDI
* Input: data to be writed
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write(char data_) {

    //Mmc_Chip_Select = 1;
    //Delay_ms(120);
    MP3_CS = 1;
    
    //while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
      BSYNC = 0;
    UART1_Write(data_);
    //SPI1_Write(data_);
   //Delay_ms(100);
    BSYNC = 1;

}

/**************************************************************************************************
* Function MP3_Reset_Time()
* -------------------------------------------------------------------------------------------------
* Overview: Function resets decode time of the current song
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Reset_Time(){
    MP3_SCI_Write(SCI_DECODE_TIME_ADDR, 0);
}

/**************************************************************************************************
* Function MP3_Get_Decode_Time()
* -------------------------------------------------------------------------------------------------
* Overview: Function resets decode time of the current song
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Get_Decode_Time(unsigned int *time){
    MP3_SCI_Read(SCI_DECODE_TIME_ADDR, 1, time);
}

/**************************************************************************************************
* Function MP3_SDI_Write_32
* -------------------------------------------------------------------------------------------------
* Overview: Function Write 32 bytes to MP3 SDI
* Input: data buffer
* Output: Nothing
**************************************************************************************************/
void MP3_SDI_Write_32(char *data_) {
    char i;

    Mmc_Chip_Select = 1;
    MP3_CS = 1;


    while (DREQ == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
      BSYNC = 0;
    for(i = 0; i < 32; i++)

        SPI1_Write(data_[i]);

    BSYNC = 1;
}

/**************************************************************************************************
* Function MP3_Set_Volume()
* -------------------------------------------------------------------------------------------------
* Overview: Function set volume on the left and right channel
* Input: left channel volume, right channel volume
* Output: Nothing
**************************************************************************************************/
void MP3_Set_Volume(char left, char right) {
    unsigned int volume;

    volume = (left << 8) + right;             // calculate value
    MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

void MP3_Set_default_Mode(){
    BSYNC = 1;
    MP3_CS = 1;

    // Hardware reset
    MP3_RST = 0;
    Delay_ms(20);
    MP3_RST = 1;

    while (DREQ == 0);

    MP3_SCI_Write(SCI_MODE_ADDR, 0x000);
    MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
    MP3_SCI_Write(SCI_CLOCKF_ADDR, 0xC000);   // default 12 288 000 Hz, PLL enable
    MP3_SCI_Write(SCI_VOL_ADDR,0x0000);
    // Maximum volume is 0x00 and total silence is 0xFE.
}
/*******************************************************************************
* Function Name  : VsSineTest
* Description    : VS1053 sine test
* Input          : None.
* Output         : None
* Return         : None
*******************************************************************************/
void VsSineTest(void)

{

        UART_Write_Text("VS Sine Test Begin!\n");

         MP3_RST = 0;  //xReset = 0   ??vs1053

        Delay_ms(100);//wait(100);        //??100ms
        SPI1_Write(0xFF);

        MP3_CS = 1;
        BSYNC = 1;

          MP3_RST = 1; //release from reset


          //MP3_RST = 0;
        Delay_ms(500);
        UART_Write_Text("SET VOLUME BEGIN Line226!\n");
        MP3_Set_Volume(0,0);
        LED = 1;
        Delay_ms(100);
        LED = 0;
        
        // MP3_Set_default_Mode();

         MP3_CS = 0;
         Mmc_Chip_Select = 1;

         MP3_SCI_Write(SCI_MODE_ADDR, 0x0820); //allow test
        Delay_ms(150);
        MP3_CS = 1;
          while (DREQ == 0);
       BSYNC  = 0; //xDCS = 0
        SPI1_Write(0x53);
        SPI1_Write(0xEF);
        SPI1_Write(0x6E);
        SPI1_Write(0x44);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        Delay_ms(100);
          BSYNC = 1;
         Delay_ms(120);
          BSYNC = 0;
       SPI1_Write(0x45);
        SPI1_Write(0x78);
        SPI1_Write(0x69);
        SPI1_Write(0x74);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        SPI1_Write(0x00);
        Delay_ms(100);
        BSYNC = 1;
        Delay_ms(500);
        UART_Write_Text("VS Sine Test END!\r\n");
 }
/**************************************************************************************************
* End of File
**************************************************************************************************/